from pytest_check import check_func

# properly formatted tests in
# multiline strings contain blank lines with spaces, freaks out flake8

# flake8: noqa

@check_func
def is_five(a):
    assert a == 5


def test_pass():
    is_five(5)


def test_return_val_True():
    assert True == is_five(5)


def test_fail(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check_func
        
        @check_func
        def is_four(a):
            assert a == 4

        def test_all_four():
            is_four(1)
            is_four(2)
            should_be_False = is_four(3)
            should_be_True = is_four(4)
            print('should_be_True={}'.format(should_be_True))
            print('should_be_False={}'.format(should_be_False))
    """
    )

    result = testdir.runpytest("-s")
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines([
        "*should_be_True=True*",
        "*should_be_False=False*",
        "*FAILURE: assert 1 == 4*",
        "*FAILURE: assert 2 == 4*",
        "*FAILURE: assert 3 == 4*",
        "*Failed Checks: 3*",
    ])



from pytest_check import check

# properly formatted tests in
# multiline strings contain blank lines with spaces, freaks out flake8

# flake8: noqa

def test_context_manager():
    with check:
        x = 3
        assert 1 < x < 4

def test_context_manager_with_msg():
    assert check.msg is None
    with check("Hello"):
        assert check.msg == "Hello"
        x = 3
        assert 1 < x < 4
    assert check.msg is None

def test_context_manager_fail(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check

        def test_failures():
            with check: assert 1 == 0
            with check: assert 1 > 2
            with check: assert 1 < 5 < 4
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines([
        "*FAILURE: assert 1 == 0*",
        "*FAILURE: assert 1 > 2*",
        "*FAILURE: assert 5 < 4*",
        "* assert 1 < 5 < 4*",
        "*Failed Checks: 3*",
    ])


def test_context_manager_fail_with_msg(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check

        def test_failures():
            with check("first fail"): assert 1 == 0
            with check("second fail"): assert 1 > 2
            with check("third fail"): assert 1 < 5 < 4
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines([
        "*FAILURE: first fail*",
        "*FAILURE: second fail*",
        "*FAILURE: third fail*",
        "* assert 1 < 5 < 4*",
        "*Failed Checks: 3*",
    ])


def test_stop_on_fail(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check

        def test_failures():
            with check: assert 1 == 0
            with check: assert 1 > 2
            with check: assert 1 < 5 < 4
    """
    )

    result = testdir.runpytest('-x')
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines([
        "*assert 1 == 0*",
    ])


def test_stop_on_fail_with_msg(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check

        def test_failures():
            with check("first fail"): assert 1 == 0
            with check("second fail"): assert 1 > 2
            with check("third fail"): assert 1 < 5 < 4
    """
    )

    result = testdir.runpytest('-x')
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines([
        "*first fail*",
    ])



def test_setup_failure(testdir):
    testdir.makepyfile(
        """
        import pytest
        import pytest_check as check

        @pytest.fixture()
        def a_fixture():
            check.equal(1, 2)

        def test_1(a_fixture):
            pass
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(errors=1)
    result.stdout.fnmatch_lines(["* check.equal(1, 2)*"])


def test_teardown_failure(testdir):
    testdir.makepyfile(
         """
         import pytest
         import pytest_check as check

         @pytest.fixture()
         def a_fixture():
             yield
             check.equal(1, 2)

         def test_1(a_fixture):
             pass
         """
    )
    result = testdir.runpytest()
    result.assert_outcomes(passed=1, errors=1)
    result.stdout.fnmatch_lines(["* check.equal(1, 2)*"])


def test_mix(testdir):
    testdir.makepyfile(
        """
        from pytest_check import check

        def test_fail_and_error(check):
            check.equal(1, 2)
            assert 2 == 3
        """
    )
    result = testdir.runpytest()
    result.assert_outcomes(failed=1, passed=0)
    result.stdout.fnmatch_lines(["* check.equal(1, 2)*",
                                 "* assert 2 == 3*"])



import functools
import inspect
import os
import pytest


__all__ = [
    "check",
    "equal",
    "not_equal",
    "is_",
    "is_not",
    "is_true",
    "is_false",
    "is_none",
    "is_not_none",
    "is_in",
    "is_not_in",
    "is_instance",
    "is_not_instance",
    "almost_equal",
    "not_almost_equal",
    "greater",
    "greater_equal",
    "less",
    "less_equal",
    "check_func"
]


_stop_on_fail = False
_failures = []


def clear_failures():
    global _failures
    _failures = []


def get_failures():
    return _failures


def set_stop_on_fail(stop_on_fail):
    global _stop_on_fail
    _stop_on_fail = stop_on_fail


class CheckContextManager(object):

    msg = None

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        __tracebackhide__ = True
        if exc_type is not None and issubclass(exc_type, AssertionError):
            if _stop_on_fail:
                self.msg = None
                return
            else:
                if self.msg is not None:
                    log_failure(self.msg)
                else:
                    log_failure(exc_val)
                self.msg = None
                return True
        self.msg = None

    def __call__(self, msg=None):
        self.msg = msg
        return self


check = CheckContextManager()


def check_func(func):
    @functools.wraps(func)
    def wrapper(*args, **kwds):
        __tracebackhide__ = True
        try:
            func(*args, **kwds)
            return True
        except AssertionError as e:
            if _stop_on_fail:
                raise e
            log_failure(e)
            return False

    return wrapper


@check_func
def equal(a, b, msg=""):
    assert a == b, msg


@check_func
def not_equal(a, b, msg=""):
    assert a != b, msg


@check_func
def is_(a, b, msg=""):
    assert a is b, msg


@check_func
def is_not(a, b, msg=""):
    assert a is not b, msg


@check_func
def is_true(x, msg=""):
    assert bool(x), msg


@check_func
def is_false(x, msg=""):
    assert not bool(x), msg


@check_func
def is_none(x, msg=""):
    assert x is None, msg


@check_func
def is_not_none(x, msg=""):
    assert x is not None, msg


@check_func
def is_in(a, b, msg=""):
    assert a in b, msg


@check_func
def is_not_in(a, b, msg=""):
    assert a not in b, msg


@check_func
def is_instance(a, b, msg=""):
    assert isinstance(a, b), msg


@check_func
def is_not_instance(a, b, msg=""):
    assert not isinstance(a, b), msg


@check_func
def almost_equal(a, b, rel=None, abs=None, msg=""):
    """
    for rel and abs tolerance, see:
    See https://docs.pytest.org/en/latest/builtin.html#pytest.approx
    """
    assert a == pytest.approx(b, rel, abs), msg


@check_func
def not_almost_equal(a, b, rel=None, abs=None, msg=""):
    """
    for rel and abs tolerance, see:
    See https://docs.pytest.org/en/latest/builtin.html#pytest.approx
    """
    assert a != pytest.approx(b, rel, abs), msg


@check_func
def greater(a, b, msg=""):
    assert a > b, msg


@check_func
def greater_equal(a, b, msg=""):
    assert a >= b, msg


@check_func
def less(a, b, msg=""):
    assert a < b, msg


@check_func
def less_equal(a, b, msg=""):
    assert a <= b, msg


def get_full_context(level):
    (_, filename, line, funcname, contextlist) = inspect.stack()[level][0:5]
    filename = os.path.relpath(filename)
    context = contextlist[0].strip()
    return (filename, line, funcname, context)


def log_failure(msg):
    __tracebackhide__ = True
    level = 3
    pseudo_trace = []
    func = ""
    while "test_" not in func:
        (file, line, func, context) = get_full_context(level)
        if "site-packages" in file:
            break
        line = "{}:{} in {}() -> {}".format(file, line, func, context)
        pseudo_trace.append(line)
        level += 1
    pseudo_trace_str = "\n".join(reversed(pseudo_trace))
    entry = "FAILURE: {}\n{}".format(msg if msg else "", pseudo_trace_str)
    _failures.append(entry)



# -*- coding: utf-8 -*-

import pytest
from . import check_methods

# This is ugly.
# But it seems to be the only way to know about xfail status.
from _pytest.skipping import xfailed_key


@pytest.hookimpl(hookwrapper=True, trylast=True)
def pytest_runtest_makereport(item, call):
    outcome = yield
    report = outcome.get_result()

    failures = check_methods.get_failures()
    check_methods.clear_failures()

    if failures:
        if item._store[xfailed_key]:
            report.outcome = "skipped"
            report.wasxfail = item._store[xfailed_key].reason
        else:

            summary = "Failed Checks: {}".format(len(failures))
            longrepr = ["\n".join(failures)]
            longrepr.append("-" * 60)
            longrepr.append(summary)

            if report.longrepr:
                longrepr.append("-" * 60)
                longrepr.append(report.longreprtext)
                report.longrepr = "\n".join(longrepr)
            else:
                report.longrepr = "\n".join(longrepr)
            report.outcome = "failed"


def pytest_configure(config):
    check_methods.set_stop_on_fail(config.getoption("-x"))


@pytest.fixture(name='check')
def check_fixture():
    return check_methods



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['pytest_check']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['pytest>=6.0.0']

extras_require = \
{'test': ['tox']}

entry_points = \
{'pytest11': ['check = pytest_check.plugin']}

setup(name='pytest_check',
      version='0.4.1',
      description='A pytest plugin that allows multiple failures per test.',
      author='Brian Okken',
      author_email='brian+pypi@pythontest.com',
      url='https://github.com/okken/pytest-check',
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      entry_points=entry_points,
     )



"""A pytest plugin that allows multiple failures per test."""
import pytest
from pytest_check.check_methods import *  # noqa: F401, F402, F403

pytest.register_assert_rewrite("pytest_check.check")

__version__ = "0.4.1"



# -*- coding: utf-8 -*-

# properly formatted tests in
# multiline strings contain blank lines with spaces, freaks out flake8

# flake8: noqa

import pytest_check as check


def test_equal():
    check.equal(1, 1)


def test_not_equal():
    check.not_equal(1, 2)


def test_is():
    x = ["foo"]
    y = x
    check.is_(x, y)


def test_is_not():
    x = ["foo"]
    y = ["foo"]
    check.is_not(x, y)


def test_is_true():
    check.is_true(True)


def test_is_false():
    check.is_false(False)


def test_is_none():
    a = None
    check.is_none(a)


def test_is_not_none():
    a = 1
    check.is_not_none(a)


def test_is_in():
    check.is_in(2, [1, 2, 3])


def test_is_not_in():
    check.is_not_in(4, [1, 2, 3])


def test_is_instance():
    check.is_instance(1, int)


def test_is_not_instance():
    check.is_not_instance(1, str)


def test_almost_equal():
    check.almost_equal(1, 1)
    check.almost_equal(1, 1.1, abs=0.2)
    check.almost_equal(2, 1, rel=1)


def test_not_almost_equal():
    check.not_almost_equal(1, 2)
    check.not_almost_equal(1, 2.1, abs=0.1)
    check.not_almost_equal(3, 1, rel=1)


def test_greater():
    check.greater(2, 1)


def test_greater_equal():
    check.greater_equal(2, 1)
    check.greater_equal(1, 1)


def test_less():
    check.less(1, 2)


def test_less_equal():
    check.less_equal(1, 2)
    check.less_equal(1, 1)


def test_watch_them_all_fail(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        
        def test_equal():
            check.equal(1,2)
            
        def test_not_equal():
            check.not_equal(1,1)
        
        def test_is():
            x = ["foo"]
            y = ["foo"]
            check.is_(x, y)
            
        def test_is_not():
            x = ["foo"]
            y = x
            check.is_not(x, y)
        
        def test_is_true():
            check.is_true(False)

            
        def test_is_false():
            check.is_false(True)
            
        
        def test_is_none():
            a = 1
            check.is_none(a)
        
        
        def test_is_not_none():
            a = None
            check.is_not_none(a)
        
        
        def test_is_in():
            check.is_in(4, [1, 2, 3])
        
            
        def test_is_not_in():
            check.is_not_in(2, [1, 2, 3])
        
        
        def test_is_instance():
            check.is_instance(1, str) 
        
        
        def test_is_not_instance():
            check.is_not_instance(1, int) 
            
        def test_almost_equal():
            check.almost_equal(1, 2) 
            check.almost_equal(1, 2.1, abs=0.1) 
            check.almost_equal(1, 3, rel=1) 
        
        
        def test_not_almost_equal():
            check.not_almost_equal(1, 1) 
            check.not_almost_equal(1, 1.1, abs=0.1) 
            check.not_almost_equal(1, 2, rel=1) 
        
        
        def test_greater():
            check.greater(1, 2) 
            check.greater(1, 1) 
        
        
        def test_greater_equal():
            check.greater_equal(1, 2) 
        
        
        def test_less():
            check.less(2, 1) 
            check.less(1, 1) 
        
        
        def test_less_equal():
            check.less_equal(2, 1) 
            #check.equal(2, 1) 
            
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(failed=18, passed=0)


def test_check_xfail(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        import pytest
        
        @pytest.mark.xfail()
        def test_fail():
            check.equal(1, 2)
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(xfailed=1)
    result.stdout.fnmatch_lines(["* 1 xfailed *"])

def test_check_xfail_strict(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        import pytest
        
        @pytest.mark.xfail(strict=True)
        def test_fail():
            check.equal(1, 2)
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(xfailed=1)
    result.stdout.fnmatch_lines(["* 1 xfailed *"])


def test_check_xpassed(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        import pytest
        
        @pytest.mark.xfail()
        def test_fail():
            check.equal(1, 1)
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(xpassed=1)

    result.stdout.fnmatch_lines(["* 1 xpassed *"])


def test_check_xpassed_strict(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        import pytest

        @pytest.mark.xfail(strict=True)
        def test_pass():
            check.equal(1, 1)
    """
    )

    result = testdir.runpytest()

    result.assert_outcomes(failed=1)

    result.stdout.fnmatch_lines(["* 1 failed *"])

def test_check_and_assert(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        import pytest
        
        def test_fail_check():
            check.equal(1, 2)
        
        def test_fail_assert():
            assert 1 == 2
    """
    )

    result = testdir.runpytest()
    result.assert_outcomes(failed=2)
    result.stdout.fnmatch_lines(["* 2 failed *"])


def test_stop_on_fail(testdir):
    testdir.makepyfile(
        """
        import pytest_check as check
        
        class TestStopOnFail():

            def test_1(self):
                check.equal(1, 1)
                check.equal(1, 2)
                check.equal(1, 3)


            def test_2(self):
                check.equal(1, 1)
                check.equal(1, 2)
                check.equal(1, 3)
    """
    )

    result = testdir.runpytest("-x")
    result.assert_outcomes(failed=1)
    result.stdout.fnmatch_lines(["*> * check.equal(1, 2)*"])
